@@ -1,5 +1,5 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Chooser.pm,v 1.25 2005/02/27 23:32:02 nekral-guest Exp $
+# $Id: Chooser.pm,v 1.28 2005/11/17 23:44:33 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -18,7 +18,6 @@ package Locale::Po4a::Chooser;
 use 5.006;
 use strict;
 use warnings;
-use Locale::gettext;
 use Locale::Po4a::Common;
 
 sub new {
@@ -35,15 +34,18 @@ sub new {
         $modname = 'NewsDebian';
     } elsif ($module eq 'latex') {
         $modname = 'LaTeX';
+    } elsif ($module eq 'text') {
+        $modname = 'Text';
     } else {
         $modname = ucfirst($module);
     }
     if (! UNIVERSAL::can("Locale::Po4a::$modname", 'new')) {
         eval qq{use Locale::Po4a::$modname};
         if ($@) {
+            my $error=$@;
             warn wrap_msg(gettext("Unknown format type: %s."), $module);
 	    warn wrap_mod("po4a::chooser",
-		gettext("Module loading error: %s"), $@)
+		gettext("Module loading error: %s"), $error)
 	      if defined $options{'verbose'} && $options{'verbose'} > 0;
             list(1);
         }
