@@ -107,13 +107,94 @@ pod, one more time).
 
 These are this module's particular options:
 
-=over 4
+=over
 
 =item B<debug>
 
 Activate debugging for some internal mechanisms of this module.
 Use the source to see which parts can be debugged.
 
+=item B<verbose>
+
+Increase verbosity.
+
+=item B<groff_code>
+
+This option permits to change the behavior of the module when it encounter
+a .de, .ie or .if section. It can take the following values:
+
+=over
+
+=item I<fail>
+
+This is the default value.
+The module will fail when a .de, .ie or .if section is encountered.
+
+=item I<verbatim>
+
+Indicates that the .de, .ie or .if sections must be copied as is
+from the original to the translated document.
+
+=item I<translate>
+
+Indicates that the .de, .ie or .if sections will be proposed for the
+translation.
+You should only use this option if a translatable string is
+contained in one of these section. Otherwise, I<verbatim>
+should be preferred.
+
+=back
+
+=item B<untranslated>
+
+=item B<noarg>
+
+=item B<translate_joined>
+
+=item B<translate_each>
+
+These options permit to specify the behavior of a new macro (defined with
+a .de request), or of a macro not supported by po4a.
+They take in argument a coma separated list of macros.
+For example:
+
+ -o noarg=FO,OB,AR -o translate_joined=BA,ZQ,UX
+
+Note: if a macro is not supported by po4a and if you consider that it is a
+standard roff macro, you should submit it to the po4a development team.
+
+B<untranslated> indicates that this macro (at its arguments) don't have to
+be translated.
+
+B<noarg> is like B<untranslated>, except that po4a will verify that no
+argument is added to this macro.
+
+B<translate_joined> indicates that po4a must propose to translate the
+arguments of the macro.
+
+With B<translate_each>, the arguments will also be proposed for the
+translation, except that each one will be translated separately.
+
+=item B<no_wrap>
+
+This option takes in argument a list of coma-separated couples
+I<begin>:I<end>, where I<begin> and I<end> are commands that delimit
+the begin and end of a section that should not be rewrapped.
+
+Note: no test is done to ensure that an I<end> command matches its
+I<begin> command; any ending command stop the no_wrap mode.
+If you have a I<begin> (respectively I<end>) macro that has no I<end>
+(respectively I<begin>), you can specify an existing I<end> (like fi) or
+I<begin> (like nf) as a counterpart.
+These macros (and their arguments) wont be translated.
+
+=item inline
+
+This option permits to specify a list of coma-separated macros that must
+not split the current paragraph. The string to translate will then contain
+I<foo E<lt>.bar baz quxE<gt> quux>, where I<foo> is the command that
+should be inlined.
+
 =back
 
 =head1 AUTHORING MAN PAGES COMPLIANT WITH PO4A::MAN
@@ -228,7 +309,7 @@ man page.
 
 Parser fails on pages based on mdoc(7), pages using conditionals with .if,
 defining new macros with .de, using non standard fonts, and more generally,
-not following the advices of previous section.
+not following the advices of the previous section.
 
 Pages with undetected problems are processed without complain by po4a::man,
 but the generated output is different from the original one (some strings
@@ -274,13 +355,11 @@ use strict;
 use warnings;
 
 require Exporter;
-use vars qw($VERSION @ISA @EXPORT);
-$VERSION=$Locale::Po4a::TransTractor::VERSION;
+use vars qw(@ISA @EXPORT);
 @ISA = qw(Locale::Po4a::TransTractor);
 @EXPORT = qw();#  new initialize);
 use Locale::Po4a::TransTractor;
 use Locale::Po4a::Common;
-use Locale::gettext qw(dgettext);
 
 use File::Spec;
 use Getopt::Std;
@@ -310,12 +389,38 @@ my %debug=('splitargs' => 0, # see how macro args are separated
 
 
 ######## CONFIG #########
+# This variable indicates the behavior of the module when a .de, .if or
+# .ie is encountered.
+my $groff_code = "fail";
+# %no_wrap_begin and %no_wrap_end are lists of macros that respectively
+# begins and ends a no_wrap paragraph.
+# Any ending macro will end the no_wrap paragraph started by any beginning
+# macro.
+my %no_wrap_begin = (
+    'nf' => 1,
+    'EX' => 1,
+    'EQ' => 1
+);
+my %no_wrap_end = (
+    'fi' => 1,
+    'EE' => 1,
+    'EN' => 1
+);
+my %inline = (
+);
 sub initialize {
     my $self = shift;
     my %options = @_;
 
     $self->{options}{'debug'}='';
     $self->{options}{'verbose'}='';
+    $self->{options}{'groff_code'}='';
+    $self->{options}{'untranslated'}='';
+    $self->{options}{'noarg'}='';
+    $self->{options}{'translate_joined'}='';
+    $self->{options}{'translate_each'}='';
+    $self->{options}{'no_wrap'}='';
+    $self->{options}{'inline'}='';
 
     # reset the debug options
     %debug = ();
@@ -334,6 +439,50 @@ sub initialize {
             $debug{$_} = 1;
         }
     }
+
+    if (defined $options{'groff_code'}) {
+        unless ($options{'groff_code'} =~ m/fail|verbatim|translate/) {
+          die wrap_mod("po4a::man", dgettext("po4a",
+               "Invalid 'groff_code' value. Must be one of 'fail', 'verbatim', 'translate'."));
+        }
+        $groff_code = $options{'groff_code'};
+    }
+
+    if (defined $options{'untranslated'}) {
+        foreach (split(/,/, $options{'untranslated'})) {
+            $macro{$_} = \&untranslated;
+        }
+    }
+    if (defined $options{'noarg'}) {
+        foreach (split(/,/, $options{'noarg'})) {
+            $macro{$_} = \&noarg;
+        }
+    }
+    if (defined $options{'translate_joined'}) {
+        foreach (split(/,/, $options{'translate_joined'})) {
+            $macro{$_} = \&translate_joined;
+        }
+    }
+    if (defined $options{'translate_each'}) {
+        foreach (split(/,/, $options{'translate_each'})) {
+            $macro{$_} = \&translate_each;
+        }
+    }
+    if (defined $options{'no_wrap'}) {
+        foreach (split(/,/, $options{'no_wrap'})) {
+            if ($_ =~ m/^(.*):(.*)$/) {
+               $no_wrap_begin{$1} = 1;
+               $no_wrap_end{$2} = 1;
+            } else {
+                die wrap_mod("po4a::man", dgettext("po4a","The no_wrap parameters must be a set of comma-separated begin:end couples.\n"));
+            }
+        }
+    }
+    if (defined $options{'inline'}) {
+        foreach (split(/,/, $options{'inline'})) {
+            $inline{$_} = 1;
+        }
+    }
 }
 
 my @comments = ();
@@ -358,6 +507,12 @@ NEW_LINE:
         return ($line,$ref);
     }
 
+    # Do as few treatments as possible with the .de, .ie and .if sections
+    if ($line =~ /^\.\s*(if|ie|de)/) {
+        chomp $line;
+        return ($line,$ref);
+    }
+
     # Handle some escapes
     #   * reduce the number of \ in macros
     if ($line =~ /^\\?[.']/) {
@@ -431,14 +586,36 @@ NEW_LINE:
         my ($l2,$r2)=$self->SUPER::shiftline();
         chomp($l2);
         if ($line =~ /^(\.[BI])\s*$/) {
-            # convert " to the groff's double quote glyph; it will be
-            # converted back to " in pre_trans. It is needed because
-            # otherwise, these quotes will be taken as arguments
-            # delimiters.
-            $l2 =~ s/"/\\(dq/g;
-            # append this line to the macro, with surrounding quotes, so
-            # that the line appear as an uniq argument.
-            $line .= ' "'.$l2.'"';
+            if (   $l2 =~ /^[.'][\t ]*([BIR]|BI|BR|IB|IR|RB|RI|SH|TP)[\t ]/
+                or $l2 =~ /^[.'][\t ]*([BIR])$/) {
+                # another font macro. Forget about the first one
+                $line = $l2;
+                $ref = $r2;
+            } elsif ($l2 =~ /^([.'][\t ]*(?:IP)[\t ]+"?)(.*)$/) {
+                # Install the font modifier into the next line
+                # after a possible quote (")
+                my $macro = $1;
+                my $arg   = $2;
+                $line =~ /^\.([BI])\s*$/;
+                $line = $macro."\\f$1".$arg;
+                $ref = $r2;
+            } elsif ($l2 =~ /^[.']/) {
+                die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
+                        "po4a does not support font modifiers followed by a ".
+                        "command.  You should either remove the font modifier ".
+                        "'%s', or integrate a \\f font modifier in the ".
+                        "following command ('%s')"
+                        ), $line, $l2);
+            } else {
+                # convert " to the groff's double quote glyph; it will be
+                # converted back to " in pre_trans. It is needed because
+                # otherwise, these quotes will be taken as arguments
+                # delimiters.
+                $l2 =~ s/"/\\(dq/g;
+                # append this line to the macro, with surrounding quotes, so
+                # that the line appear as an uniq argument.
+                $line .= ' "'.$l2.'"';
+            }
         } else {
             $line =~ s/\\$//;
             $line .= $l2;
@@ -528,7 +705,7 @@ sub pushmacro {
 			length($_)?
 			    (m/([^\\] |^ )/ ? "\"$_\"" : "$_")
 			    # Quote arguments that contain a space.
-			    # (not needd for non breaknig spaces, i.e.
+			    # (not needed for non breaknig spaces, i.e.
 			    # spaces preceded by '\')
 			    :'""' # empty argument
 		) : '' # no argument
@@ -557,6 +734,11 @@ sub pre_trans {
     print STDERR "pre_trans($str)="
 	if ($debug{'pretrans'});
 
+    # Do as few treatments as possible with the .de, .ie and .if sections
+    if (defined $self->{type} && $self->{type} =~ m/^(ie|if|de)$/) {
+        return $str;
+    }
+
     # Note: if you want to implement \c support, the gdb man page is your playground
     die wrap_ref_mod($ref, "po4a::man", dgettext("po4a","Escape sequence \\c encountered. This is not completely handled yet."))
 	if ($str =~ /\\c/);
@@ -565,6 +747,15 @@ sub pre_trans {
     $str =~ s/</E<lt>/sg;
     $str =~ s/EE<lt>gt>/E<gt>/g; # could be done in a smarter way?
 
+    while ($str =~ m/^(.*)PO4A-INLINE:(.*?):PO4A-INLINE(.*)$/s) {
+        my ($t1,$t2, $t3) = ($1, $2, $3);
+        $str = "$1E<$2>";
+        if (defined $t3 and length $t3) {
+            $t3 =~ s/^\n//s;
+            $str .= "\n$t3";
+        }
+    }
+
     # simplify the fonts for the translators
     if (defined $self->{type} && $self->{type} =~ m/^(SH|SS)$/) {
         set_regular("B");
@@ -605,11 +796,16 @@ sub pre_trans {
 }
 
 sub post_trans {
-    my ($self,$str,$ref,$type)=@_;
+    my ($self,$str,$ref,$type,$wrap)=@_;
     my $transstr=$str;
 
     print STDERR "post_trans($str)="
 	if ($debug{'postrans'});
+    
+    # Do as few treatments as possible with the .de, .ie and .if sections
+    if (defined $self->{type} && $self->{type} =~ m/^(ie|if|de)$/) {
+        return $str;
+    }
 
     # Post formatting, so that groff see the strange chars
     $str =~ s|\\-|-|sg; # in case the translator added some of them manually
@@ -617,21 +813,45 @@ sub post_trans {
     # (this shouldn't be done for \s-<number> font size modifiers)
     # nor on .so/.mso args
     unless (defined $self->{type} && $self->{type} =~ m/^m?so$/) {
-        $str =~ s/(?<!\\s)-/\\-/sg; # (?<!pattern) means "not preceded by pattern"
+        my $tmp = "";
+        while ($str =~ m/^(.*?)-(.*)$/s) {
+            my $begin = $1;
+            $str = $2;
+            my $tmp2 = $tmp.$begin;
+            if (   ($begin =~ m/\\s$/s)
+                or ($begin =~ m/\\\((.|E<[gl]t>)$/s)
+                or ($tmp2 =~ m/\\h'([^']|\\')*$/)) {
+                # Do not change - to \- for
+                #  * \s-n (reduce font size)
+                #  * \(.- (a character named '.-', e.g. '<-')
+                #  * inside a \h'...'
+                $tmp = $tmp2."-";
+            } else {
+                $tmp = $tmp2."\\-";
+            }
+        }
+        $str = $tmp.$str;
     }
 
     # No . or ' on first char, or nroff will think it's a macro
     # * at the beginning of a paragraph, add \& (zero width space) at
     #   the beginning of the line
-    unless (defined $self->{type} && $self->{type} =~ m/^(TS)$/) {
-        # This doesn't work after a TS (this macros shift
+    if (not defined $self->{type}) {
+        # Only do it on regular text, because
+        # his doesn't work after a TS (this macros shift
         # lines, which may contain macros)
+        # or for the .ta arguments (e.g. .ta .5i 3i)
         $str =~ s/^((?:
                        (?:CW|[RBI])<
                       |$FONT_RE
                     )?
                     [.']
                    )/\\&$1/mgx;
+    } elsif ($self->{type} =~ m/^(TP)$/) {
+        # But it is also needed for some type (e.g. TP, if followed by a
+        # font macro)
+        # This regular expression is the same as above
+        $str =~ s/^((?:(?:CW|[RBI])<|$FONT_RE)?[.'])/\\&$1/mg;
     }
     # * degraded mode, doesn't work for the first line of a paragraph
     $str =~ s/\n([.'])/ $1/mg;
@@ -661,13 +881,33 @@ sub post_trans {
 	    $done .= $first  if ($lvl > 0);
 	    $rest=substr($rest,1);
 	}
-	die wrap_ref_mod($ref||$self->{ref}, "po4a::man", dgettext("po4a","Unbalanced '<' and '>'"))
+	die wrap_ref_mod($ref||$self->{ref}, "po4a::man", dgettext("po4a","Unbalanced '<' and '>' in font modifier. Faulty message: %s"),$str)
 	    if ($lvl > 0);
 	# Return to the regular font
 	$done .= "\\fP$rest";
 	$str=$done;
     }
 
+    while ($str =~ m/^(.*?)E<([.'][\t ]*.*?)(?<!E<[gl]t)>(.*)$/s) {
+        my ($t1, $t2, $t3) = ($1,$2,$3);
+        $t1 =~ s/ +$//s;
+        $t2 =~ s/\n/ /gs;
+        $t3 =~ s/^ +//s;
+        if ($wrap) {
+        # The no-wrap case should be checked
+            $t1 =~ s/\n$//s;
+        }
+        $str = $t1;
+        if (length $t1) {
+            $t1 =~ s/\n$//s;
+            $str = "$t1\n";
+        }
+        $str .= $t2;
+        if (defined $t3 and length $t3) {
+            $t3 =~ s/^\n//s;
+            $str.= "\n$t3";
+        }
+    }
     $str =~ s/E<gt>/>/mg;
     $str =~ s/E<lt>/</mg;
     # Don't do that, because we'll go into trouble if previous line was .TP
@@ -702,7 +942,7 @@ sub translate {
 	}
 	$str = join("\n",@paragraph)."\n";
     }
-    $str=post_trans($self,$str,$ref||$self->{ref},$type);
+    $str=post_trans($self,$str,$ref||$self->{ref},$type, $options{'wrap'});
     return $str;
 }
 
@@ -775,11 +1015,22 @@ sub parse{
 	    my $macro=$2;
 	    my $arguments=$3;
 
+	    if ($inline{$macro}) {
+		$paragraph .= "PO4A-INLINE:".$line.":PO4A-INLINE\n";
+		goto LINE;
+	    }
+
 	    # Split on spaces for arguments, but not spaces within double quotes
 	    my @args=();
 	    $line =~ s/\\ /$nbs/g; # This is probably not needed
 	    push @args,$arg1;
-	    push @args, splitargs($ref,$arguments);
+	    if ($macro =~ /^(?:ta|TP)$/) {
+		# The number of spaces may be critical for the 'ta' macro,
+		# and there is no need to split the arguments.
+		push @args, $arguments;
+	    } else {
+		push @args, splitargs($ref,$arguments)
+	    }
 
 
 	    if (length($paragraph)) {
@@ -802,9 +1053,8 @@ sub parse{
 	    # Special case:
 	    #  .nf => stop wrapped mode
 	    #  .fi => wrap again
-	    if ($macro eq 'nf' || $macro eq 'fi' ||
-                $macro eq 'EX' || $macro eq 'EE') {
-		if ($macro eq 'fi' || $macro eq 'EE') {
+	    if ($no_wrap_begin{$macro} or $no_wrap_end{$macro}) {
+		if ($no_wrap_end{$macro}) {
 		    $wrapped_mode='YES';
 		} else {
 		    $wrapped_mode='MACRONO';
@@ -1194,8 +1444,8 @@ sub translate_joined {
 
 # For macro taking several arguments, having to be translated separatly
 sub translate_each {
-   my ($self,$first)= (shift,0);
-    $self->pushmacro( map { $first++ ? $_:$self->t($_) } @_);
+    my ($self,$first)= (shift,0);
+    $self->pushmacro( map { $first++ ?$self->t($_):$_ } @_);
 }
 
 # For macro which shouldn't be given any arg
@@ -1290,16 +1540,24 @@ $macro{'TP'}=sub {
 	($l2,$ref2) = $self->shiftline();
 	chomp($l2);
     }
-    if ($l2 =~/^[.']/) {
-	# If the line after a .TP is a macro,
-	# let the parser do it's job.
-	# Note: use Transtractor unshiftline for now. This may require an
-	#       implementation of the man module's own implementation.
-	#       This may be a problem if, for example, the line resulted
-	#       of a line continuation.
-	$self->SUPER::unshiftline($l2,$ref2);
+    if ($l2 =~/^([.'][\t ]*([^\t ]*))(?:([\t ]+)(.*)$|$)/) {
+        if ($inline{$2}) {
+            my $tmp = "";
+            if (defined $4 and length $4) {
+                $tmp = $3.$self->t($4, "wrap" => 0);
+            }
+            $self->pushline($1.$tmp."\n");
+        } else {
+            # If the line after a .TP is a macro,
+            # let the parser do it's job.
+            # Note: use Transtractor unshiftline for now. This may require an
+            #       implementation of the man module's own implementation.
+            #       This may be a problem if, for example, the line resulted
+            #       of a line continuation.
+            $self->SUPER::unshiftline($l2,$ref2);
+        }
     } else {
-	$self->pushline($self->t($l2)."\n");
+	$self->pushline($self->t($l2, "wrap" => 0)."\n");
     }
 
     # From info groff:
@@ -1402,11 +1660,48 @@ $macro{'bp'}=\&untranslated;
 $macro{'ad'}=\&untranslated;
 # .de macro Define or redefine macro until .. is encountered.
 $macro{'de'}=sub {
-    die wrap_mod("po4a::man", dgettext("po4a", "This page defines a new macro with '.de'. Since po4a is not a real groff parser, this is not supported."));
+    if ($groff_code ne "fail") {
+        my $self = shift;
+        my $paragraph = "@_";
+        my $end = ".";
+        if ($paragraph=~/^[.'][\t ]*de[\t ]+([^\t ]+)[\t ]+([^\t ]+)[\t ]$/) {
+            $end = $2;
+        }
+        my ($line, $ref) = $self->SUPER::shiftline();
+        chomp $line;
+        $paragraph .= "\n".$line;
+        while (defined($line) and $line ne ".$end") {
+            ($line, $ref) = $self->SUPER::shiftline();
+            if (defined $line) {
+                chomp $line;
+                $paragraph .= "\n".$line;
+            }
+        }
+        $paragraph .= "\n";
+        if ($groff_code eq "verbatim") {
+            $self->pushline($paragraph);
+        } else {
+            $self->pushline( $self->translate($paragraph,
+                                              $self->{ref},
+                                              "groff code",
+                                              "wrap" => 0) );
+        }
+    } else {
+        die wrap_mod("po4a::man", dgettext("po4a", "This page defines a new macro with '.de'. Since po4a is not a real groff parser, this is not supported."));
+    }
 };
 # .ds stringvar anything
 #                 Set stringvar to anything.
-$macro{'ds'}=\&untranslated;
+$macro{'ds'}=sub {
+    my ($self, $m) = (shift,shift);
+    my $name = shift;
+    my $string = "@_";
+    # indicate to which variable this corresponds. The translator can
+    # find references to this string in the translation "\*(name" or
+    # "\*[name]"
+    $self->{type} = "ds $name";
+    $self->pushline($m." ".$name." ".$self->translate($string)."\n");
+};
 #       .fam      Return to previous font family.
 #       .fam name Set the current font family to name.
 $macro{'fam'}=\&untranslated;
@@ -1431,8 +1726,53 @@ $macro{'hy'}=$macro{'hym'}=$macro{'hys'}=\&untranslated;
 # .ie cond anything  If cond then anything else goto .el.
 # .if cond anything  If cond then anything; otherwise do nothing.
 $macro{'ie'}=$macro{'if'}=sub {
-    die wrap_mod("po4a::man", dgettext("po4a",
-    	"This page uses conditionals with '%s'. Since po4a is not a real groff parser, this is not supported."), $_[1]);
+    if ($groff_code ne "fail") {
+        my $self = shift;
+        my $m = $_[0];
+        my $paragraph = "@_";
+        my ($line,$ref);
+        my $count = 0;
+        $count = 1 if ($paragraph =~ m/(?<!\\)\\\{/s);
+        while (   ($paragraph =~ m/(?<!\\)\\$/s)
+               or ($count > 0)) {
+            ($line,$ref)=$self->SUPER::shiftline();
+            chomp $line;
+            $paragraph .= "\n".$line;
+            $count += 1 if ($line =~ m/(?<!\\)\\\{/s);
+            $count -= 1 if ($line =~ m/(?<!\\)\\\}/s);
+        }
+        if ($m eq '.ie') {
+            ($line,$ref)=$self->SUPER::shiftline();
+            chomp $line;
+            if ($line !~ m/^\.[ \t]*el\s/) {
+                die ".ie without .el\n"
+            }
+            my $paragraph2 = $line;
+            $count = 0;
+            $count = 1 if ($line =~ m/(?<!\\)\\\{/s);
+            while (   ($paragraph2 =~ m/(?<!\\)\\$/s)
+                   or ($count > 0)) {
+                ($line,$ref)=$self->SUPER::shiftline();
+                chomp $line;
+                $paragraph2 .= "\n".$line;
+                $count += 1 if ($line =~ m/(?<!\\)\\\{/s);
+                $count -= 1 if ($line =~ m/(?<!\\)\\\}/s);
+            }
+            $paragraph .= "\n".$paragraph2;
+        }
+        $paragraph .= "\n";
+        if ($groff_code eq "verbatim") {
+            $self->pushline($paragraph);
+        } else {
+            $self->pushline( $self->translate($paragraph,
+                                              $self->{ref},
+                                              "groff code",
+                                              "wrap" => 0) );
+        }
+    } else {
+        die wrap_mod("po4a::man", dgettext("po4a",
+            "This page uses conditionals with '%s'. Since po4a is not a real groff parser, this is not supported."), $_[1]);
+    }
 };
 # .in  N    Change indent according to N (default scaling indicator m).
 $macro{'in'}=\&untranslated;
@@ -1477,10 +1817,24 @@ $macro{'so'}= $macro{'mso'} = sub {
 # .sp     Skip one line vertically.
 # .sp N   Space  vertical distance N
 $macro{'sp'}=\&untranslated;
+# .vs [space]
+# .vs +space
+# .vs -space
+# Change (increase, decrease) the vertical spacing by SPACE.  The
+# default scaling indicator is `p'.
+$macro{'vs'}=\&untranslated;
 # .ta T N   Set tabs after every position that is a multiple of N.
 # .ta n1 n2 ... nn T r1 r2 ... rn
 #           Set  tabs at positions n1, n2, ..., nn, [...]
-$macro{'ta'}=\&untranslated;
+$macro{'ta'}=sub {
+    # In some cases, a ta request can contain a translatable argument.
+    # FIXME: detect those cases (something like 5i does not need to be
+    # translated)
+    my ($self,$m)=(shift,shift);
+    my $line = "@_";
+    $line =~ s/^ +//;
+    $self->pushline($m." ".$self->translate($line,$self->{ref},'ta')."\n");
+};
 # .ti +N    Temporary indent next line (default scaling indicator m).
 $macro{'ti'}=\&untranslated;
 
@@ -1601,3 +1955,11 @@ $macro{'ec'}=sub {
 ###        this from the generated manpage, and declare our own header
 ###
 $macro{'UC'}=$macro{'AT'}=\&untranslated;
+
+# Request: .hw word1 word2 ...
+#   Define how WORD1, WORD2, etc. are to be hyphenated.  The words
+#   must be given with hyphens at the hyphenation points.
+#
+#   If the english page needs to specify how a word must be hyphanted, the
+#   translated page may also have this need.
+$macro{'hw'}=\&translate_each;
