@@ -1,5 +1,5 @@
 # Locale::Po4a::Po -- manipulation of po files 
-# $Id: Po.pm,v 1.41 2005/04/03 21:15:22 nekral-guest Exp $
+# $Id: Po.pm,v 1.51 2005/12/04 20:25:52 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -18,7 +18,7 @@ Locale::Po4a::Po - po file manipulation module
     my $pofile=Locale::Po4a::Po->new();
 
     # Read po file
-    $pofile->load('file.po');
+    $pofile->read('file.po');
 
     # Add an entry
     $pofile->push('msgid' => 'Hello', 'msgstr' => 'bonjour', 
@@ -46,6 +46,16 @@ translate everything, including documentation (man page, info manual),
 package description, debconf templates, and everything which may benefit
 from this.
 
+=head1 OPTIONS ACCEPTED BY THIS MODULE
+
+=over 4
+
+=item porefs
+
+This specifies the reference format. It can be one of 'none' to not produce
+any reference, 'noline' to not specify the line number, and 'full' to
+include complete references.
+
 
 =cut
 
@@ -57,20 +67,18 @@ require Exporter;
 package Locale::Po4a::Po;
 
 use Locale::Po4a::TransTractor;
-use Locale::Po4a::Common;
+use Locale::Po4a::Common qw(wrap_msg wrap_mod wrap_ref_mod dgettext);
 
 use 5.006;
 use strict;
 use warnings;
 
 use subs qw(makespace);
-use vars qw($VERSION @ISA @EXPORT);
-$VERSION=$Locale::Po4a::TransTractor::VERSION;
+use vars qw(@ISA @EXPORT);
 @ISA = ();
-@EXPORT = qw(load write gettext);
+@EXPORT = qw(read write write_if_needed gettext);
 
 use Carp qw(croak);
-use Locale::gettext qw(dgettext);
 use File::Path; # mkdir before write
 
 use Encode;
@@ -95,11 +103,11 @@ a po file we should load.
 =cut
 
 sub new {
-    my $this = shift;
+    my ($this, $options) = (shift, shift);
     my $class = ref($this) || $this;
     my $self = {};
     bless $self, $class;
-    $self->initialize();
+    $self->initialize($options);
  
     my $filename = shift;
     $self->read($filename) if defined($filename) && length($filename);
@@ -107,9 +115,22 @@ sub new {
 }
 
 sub initialize {
-    my $self = shift;
+    my ($self, $options) = (shift, shift);
     my $date = `date +'%Y-%m-%d %k:%M%z'`;
     chomp $date;
+#    $options = ref($options) || $options;
+
+    $self->{options}{'porefs'}= 'full';
+    foreach my $opt (keys %$options) {
+	if ($options->{$opt}) {
+	    die wrap_mod("po4a::po", dgettext ("po4a", "Unknown option: %s"), $opt) unless exists $self->{options}{$opt};
+	    $self->{options}{$opt} = $options->{$opt};
+	}
+    }
+    $self->{options}{'porefs'} =~ /^(full|noline|none)$/ ||
+        die wrap_mod("po4a::po", 
+                     dgettext ("po4a", "Invalid value for option 'porefs' ('%s' is not one of 'full', 'noline' or 'none')"), 
+                     $self->{options}{'porefs'});
 
     $self->{po}=();
     $self->{count}=0;
@@ -282,6 +303,45 @@ sub write{
 #    }
 }
 
+=item write_if_needed($$)
+
+Like write, but if the PO or POT file already exists, the object will be
+written in a temporary file which will be compared with the existing file
+to check that the update is needed (this avoids to change a POT just to
+update a line reference or the POT-Creation-Date field).
+
+=cut
+
+sub write_if_needed {
+    my $self=shift;
+    my $filename=shift 
+	or croak (dgettext("po4a","Can't write to a file without filename")."\n");
+
+    if (-e $filename) {
+        my ($tmp_filename, $diff);
+        (undef,$tmp_filename)=File::Temp->tempfile($filename."XXXX",
+                                                   DIR    => "/tmp",
+                                                   OPEN   => 0,
+                                                   UNLINK => 0);
+        $self->write($tmp_filename);
+        $diff = qx(diff -q -I'#:' -I'POT-Creation-Date:' -I'PO-Revision-Date:' $filename $tmp_filename);
+        if ( $diff eq "" ) {
+            unlink $tmp_filename ||
+                die wrap_msg(dgettext("po4a","Can't unlink %s."),
+                             $tmp_filename);
+            # touch it
+            my ($atime, $mtime) = (time,time);
+            utime $atime, $mtime, $filename;
+        } else {
+            rename $tmp_filename, $filename ||
+                die wrap_msg(dgettext("po4a","Can't rename %s to %s."),
+                             $tmp_filename, $filename);
+        }
+    } else {
+        $self->write($filename);
+    }
+}
+
 =item gettextize($$)
 
 This function produces one translated message catalog from two catalogs, an
@@ -296,19 +356,31 @@ sub gettextize {
     my ($poorig,$potrans)=(shift,shift);
 
     my $pores=Locale::Po4a::Po->new();
+    
+    my $please_fail = 0;
+    my $toobad = dgettext("po4a",
+  	"\nThe gettextization failed (once again). Don't give up, gettextizing is a subtle art, ".
+	"but this is only needed once to convert a project to the gorgeous luxus offered ".
+        "by po4a to translators.".
+        "\nPlease refer to the po4a(7) documentation, the section \"HOWTO convert a pre-existing ".
+        "translation to po4a?\" contains several hints to help you in your task");
 
+    # Don't fail right now when the entry count does not match. Instead, give it a try so that the user
+    # can see where we fail (which is probably where the problem is).
     if ($poorig->count_entries() > $potrans->count_entries()) {
 	warn wrap_mod("po4a gettextize", dgettext("po4a",
 	    "Original has more strings than the translation (%d>%d). ".
 	    "Please fix it by editing the translated version to add some dummy entry."),
 		$poorig->count_entries() , $potrans->count_entries());
+        $please_fail = 1;      
     } elsif ($poorig->count_entries() < $potrans->count_entries()) {
 	warn wrap_mod("po4a gettextize", dgettext("po4a",
 	    "Original has less strings than the translation (%d<%d). ".
 	    "Please fix it by removing the extra entry from the translated file. ".
 	    "You may need an addendum (cf po4a(7)) to reput the chunk in place after gettextization. ".
 	    "A possible cause is that a text duplicated in the original is not translated the same way each time. Remove one of the translations, and you're fine."),
-		$poorig->count_entries(), $potrans->count_entries());
+               $poorig->count_entries(), $potrans->count_entries());
+        $please_fail = 1;
     }
 
     if ( $poorig->get_charset =~ /^utf-8$/i ) {
@@ -361,8 +433,8 @@ sub gettextize {
 		"msgstr (at %s) is of type '%s'.\n".
 		"Original text: %s\n".
 		"Translated text: %s\n".
-	        "(result so far dumped to /tmp/gettextization.failed.po)"),
-		    $reforig, $typeorig, $reftrans, $typetrans, $orig, $trans);
+	        "(result so far dumped to /tmp/gettextization.failed.po)")."%s",
+	        $reforig, $typeorig, $reftrans, $typetrans, $orig, $trans,$toobad);
 	}
 
 	# 
@@ -373,6 +445,7 @@ sub gettextize {
 	                 'type'  => $typeorig,
 			 'reference' => $reforig);
     }
+    die "$toobad\n" if $please_fail; # make sure we return a useful error message when entry count differ
     return $pores;
 }
 
@@ -404,7 +477,7 @@ sub filter {
 
     sub gloups {
 	my $fmt=shift;
-	my $space;
+	my $space = "";
 	for (1..$pos){
 	    $space .= ' ';
 	}
@@ -643,7 +716,7 @@ sub gettext {
     my $res;
 
     return "" unless defined($text) && length($text); # Avoid returning the header.
-    my $validoption="wrap wrapcol";
+    my $validoption="reference wrap wrapcol";
     my %validoption;
 
     map { $validoption{$_}=1 } (split(/ /,$validoption));
@@ -852,7 +925,13 @@ sub push_raw {
 	$self->{header_comment}=$comment;
 	return;
     }
-    
+
+    if ($self->{options}{'porefs'} eq "none") {
+        $reference = "";
+    } elsif ($self->{options}{'porefs'} eq "noline") {
+        $reference =~ s/:[0-9]*/:1/g;
+    }
+
     if (defined($self->{po}{$msgid})) {
         warn wrap_mod("po4a::po", dgettext("po4a","msgid defined twice: %s"), $msgid) if (0); # FIXME: put a verbose stuff
 	if ($msgstr && $self->{po}{$msgid}{'msgstr'} 
@@ -1102,6 +1181,11 @@ sub wrap {
 	    while (substr($line,$pos-1,1) eq '.' && $pos != -1) {
 		$pos=rindex($line," ",$pos-1);
 	    }
+	    if ($pos == -1) {
+		# There are no spaces in the first $col chars, pick-up the
+		# first space
+		$pos = index($line," ");
+	    }
 	    if ($pos != -1) {
 		my $end=substr($line,$pos+1);
 		$line=substr($line,0,$pos+1);
@@ -1114,6 +1198,11 @@ sub wrap {
 	$first=0;
 	$res.="$line\n";
     }
+    # Restore the original trailing spaces
+    chomp $res;
+    if ($text =~ m/(\s+)$/s) {
+	$res .= $1;
+    }
     return $res;
 }
 
