@@ -1,13 +1,11 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Pod.pm,v 1.13 2004/08/27 18:18:34 barbier Exp $
-#
-# Copyright 2002 by Martin Quinson <Martin.Quinson@ens-lyon.fr>
+# $Id: Pod.pm,v 1.18 2005/10/26 10:53:25 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
-# under the terms of GPL (see COPYING).
+# under the terms of GPL (see COPYING file).
 #
 # This module converts POD to PO file, so that it becomes possible to 
-# translate POD formated documentation. See gettext documentation for
+# translate POD formatted documentation. See gettext documentation for
 # more info about PO files.
 
 ############################################################################
@@ -25,8 +23,7 @@ use warnings;
 
 require Exporter;
 
-use vars qw($VERSION @ISA);
-$VERSION=$Locale::Po4a::TransTractor::VERSION;
+use vars qw(@ISA);
 @ISA = qw(Locale::Po4a::TransTractor Pod::Parser);
 
 sub initialize {}
@@ -45,7 +42,7 @@ sub command {
 				    $self->input_file().":$line_num",
 				    "=$command",
 				    "wrap"=>1);
-	$self->pushline("=$command $paragraph\n");
+	$self->pushline("=$command $paragraph\n\n");
     }
 }
 
@@ -90,7 +87,7 @@ sub textblock {
 				'textblock',
 				"wrap"=>1);
     $paragraph=~ s/ +\n/\n/gm;
-    $self->pushline("$paragraph\n");
+    $self->pushline("$paragraph\n\n");
 }
 
 sub end_pod {}
@@ -171,10 +168,10 @@ and mine contain
 
  "" #n""
  
-which is logic since C<blabla> is rewritten "blabla"
+which is logic since C<foobar> is rewritten "foobar"
 
 Complete list of pages having this problem on my box (from 564 pages ; note
-that it depend on the choosen wrapping colon):
+that it depends on the chosen wrapping column):
 /usr/lib/perl5/Tk/MainWindow.pod
 /usr/share/perl/5.8.0/overload.pod
 /usr/share/perl/5.8.0/pod/perlapi.pod
@@ -257,13 +254,13 @@ L<Locale::Po4a::Man(3pm)>,
 =head1 AUTHORS
 
  Denis Barbier <barbier@linuxfr.org>
- Martin Quinson <martin.quinson@tuxfamily.org>
+ Martin Quinson (mquinson#debian.org)
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2002 by SPI, inc.
 
 This program is free software; you may redistribute it and/or modify it
-under the terms of GPL (see COPYING file).
+under the terms of GPL (see the COPYING file).
 
 =cut
