@@ -10,7 +10,7 @@ use warnings;
 
 use subs qw(makespace);
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION="0.20";
+$VERSION="0.23";
 @ISA = ();
 @EXPORT = qw(process translate 
 	     read write readpo writepo
@@ -20,7 +20,6 @@ use Carp qw(croak);
 use Locale::Po4a::Po;
 use Locale::Po4a::Common;
 
-use Locale::gettext qw(dgettext);
 use File::Path; # mkdir before write
 
 use Encode;
@@ -113,8 +112,8 @@ sub parse {}
 
 =head1 SYNOPSIS
 
-The following example parses a list of paragraphs beginning with "<p>". For sake
-of simplicity, we assume that the document is well formatted, ie that '<p>'
+The following example parses a list of paragraphs beginning with "<p>". For the sake
+of simplicity, we assume that the document is well formatted, i.e. that '<p>'
 tags are the only tags present, and that this tag is at the very beginning
 of each paragraph.
 
@@ -152,7 +151,7 @@ of each paragraph.
            # Reinit the loop
            ($line,$lref)=$self->shiftline();
        }
-       # Did not got a defined line? End of input file.
+       # Did not get a defined line? End of input file.
        return;
    }
  } 
@@ -233,7 +232,7 @@ extracted from the input document.
 
 =item addendum (@)
 
-list of filenames where we should read the addenda from.
+List of filenames where we should read the addenda from.
 
 =item addendum_charset ($)
 
@@ -327,11 +326,20 @@ sub new {
     ## Bless ourselves into the desired class and perform any initialization
     bless $self, $class;
     
+    ## initialize the plugin
+    # prevent the plugin from croaking on the options intended for Po.pm
+    $self->{options}{'porefs'} = '';
+    # let the plugin parse the options and such
+    $self->initialize(%options);
+
     ## Create our private data
+    my %po_options;
+    $po_options{'porefs'} = $self->{options}{'porefs'};
+    
     # private data
     $self->{TT}=(); 
     $self->{TT}{po_in}=Locale::Po4a::Po->new();
-    $self->{TT}{po_out}=Locale::Po4a::Po->new();
+    $self->{TT}{po_out}=Locale::Po4a::Po->new(\%po_options);
     # Warning, this is an array of array:
     #  The document is splited on lines, and for each
     #  [0] is the line content, [1] is the reference [2] the type
@@ -349,8 +357,6 @@ sub new {
     # document isn't in ascii)
     $self->{TT}{utf_mode}=0;
 
-    ## initialize the plugin
-    $self->initialize(%options);
     
     return $self;
 }
@@ -822,6 +828,10 @@ sub translate {
 	}
     }
 
+    if ($self->{TT}{po_in}->get_charset ne "CHARSET") {
+	Encode::from_to($string, $in_charset, $self->{TT}{po_in}->get_charset);
+    }
+
     my $transstring = $self->{TT}{po_in}->gettext($string,
 					'wrap'      => $options{'wrap'}||0,
 					'wrapcol'   => $options{'wrapcol'});
@@ -856,7 +866,7 @@ sub translate {
     $self->{TT}{po_out}->push('msgid'     => $string,
 			      'reference' => $ref,
 			      'type'      => $type,
-	                      'automatic' => $options{'comment'}||"",
+	                      'automatic' => $options{'comment'},
 			      'wrap'      => $options{'wrap'}||0,
 			      'wrapcol'   => $options{'wrapcol'});
 
@@ -988,9 +998,9 @@ To address this problem, the only interface changes needed are:
 
 =over 2
 
-=item - 
+=item -
 
-take an hash as po_in_name (a list per language)
+take a hash as po_in_name (a list per language)
 
 =item -
 
@@ -1013,7 +1023,7 @@ Will see if it's enough ;)
 
  Denis Barbier <barbier@linuxfr.org>
  Martin Quinson (mquinson#debian.org)
- Jordi Vilalta <jvprat@wanadoo.es>
+ Jordi Vilalta <jvprat@gmail.com>
 
 =cut
 
