@@ -8,7 +8,7 @@
 # XML documents, and it can be used as a base to build modules for
 # XML-based documents.
 #
-# Copyright (c) 2004 by Jordi Vilalta  <jvprat@wanadoo.es>
+# Copyright (c) 2004 by Jordi Vilalta  <jvprat@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -59,7 +59,6 @@ use vars qw(@ISA @EXPORT);
 
 use Locale::Po4a::TransTractor;
 use Locale::Po4a::Common;
-use Locale::gettext qw(dgettext);
 
 #It will mantain the path from the root tag to the current one
 my @path;
@@ -132,7 +131,7 @@ way.  If it's defined, it will treat E<lt>BooKE<gt>laNG and E<lt>BOOKE<gt>Lang a
 
 =item tagsonly
 
-Extracts only the specified tags in the "translate" option.  Otherwise, it
+Extracts only the specified tags in the "tags" option.  Otherwise, it
 will extract all the tags except the ones specified.
 
 =item doctype
@@ -168,6 +167,11 @@ E<lt>aaaE<gt> tag, and it's into a E<lt>bbbE<gt> tag.
 Space-separated list of the tags you want to treat as inline.  By default,
 all tags break the sequence.  This follows the same syntax as the tags option.
 
+=item nodefault
+
+Space separated list of tags that the module should not try to set by
+default in the "tags" or "inline" category.
+
 =back
 
 =cut
@@ -184,6 +188,7 @@ sub initialize {
 	$self->{options}{'attributes'}='';
 	$self->{options}{'inline'}='';
 	$self->{options}{'doctype'}='';
+	$self->{options}{'nodefault'}='';
 
 	$self->{options}{'verbose'}='';
 	$self->{options}{'debug'}='';
@@ -203,6 +208,9 @@ sub initialize {
 	$self->{attributes}=();
 	#It will maintain the list of the inline tags
 	$self->{inline}=();
+	#list of the tags that must not be set in the tags or inline category
+	#by this module or sub-module (unless specified in an option)
+	$self->{nodefault}=();
 
 	$self->treat_options;
 }
@@ -348,7 +356,7 @@ single string.
 my @tag_types = ( 
 	{	beginning	=> "!--",
 		end		=> "--",
-		breaking	=> 1,
+		breaking	=> 0,
 		f_extract	=> \&tag_extract_comment,
 		f_translate	=> \&tag_trans_comment},
 	{	beginning	=> "?xml",
@@ -399,7 +407,15 @@ sub tag_trans_xmlhead {
 	$self->detected_charset($in_charset);
 	my $out_charset=$self->get_out_charset;
 
-	$tag =~ s/$in_charset/$out_charset/;
+	if (defined $in_charset) {
+		$tag =~ s/$in_charset/$out_charset/;
+	} else {
+		if ($tag =~ m/standalone/) {
+			$tag =~ s/(standalone)/encoding="$out_charset" $1/;
+		} else {
+			$tag.= " encoding=\"$out_charset\"";
+		}
+	}
 
 	return $tag;
 }
@@ -432,7 +448,9 @@ sub tag_trans_doctype {
 	}
 	my $i = 0;
 	while ( $i < $#tag ) {
-		if ( $tag[$i] =~ /^(<!ENTITY\s+)(.*)$/is ) {
+		my $t = $tag[$i];
+		my $ref = $tag[$i+1];
+		if ( $t =~ /^(\s*<!ENTITY\s+)(.*)$/is ) {
 			my $part1 = $1;
 			my $part2 = $2;
 			my $includenow = 0;
@@ -452,11 +470,25 @@ sub tag_trans_doctype {
 				$part2 = $2;
 				$file = 1;
 			}
+			if ((not $file) and (not $includenow)) {
+			    if ($part2 =~ m/^\s*(["'])(.*)\1(\s*>.*)$/s) {
+				my $comment = "Content of the $name entity";
+				my $quote = $1;
+				my $text = $2;
+				$part2 = $3;
+				$text = $self->translate($text,
+				                         $ref,
+				                         $comment,
+				                         'wrap'=>1);
+				$t = $part1."$quote$text$quote$part2";
+			    }
+			}
 #			print $part1."\n";
 #			print $name."\n";
 #			print $part2."\n";
 		}
-		$i++;
+		$tag[$i] = $t;
+		$i += 2;
 	}
 	return $self->join_lines(@tag);
 }
@@ -823,18 +855,94 @@ sub treat_attributes {
 sub treat_content {
 	my $self = shift;
 	my $blank="";
+	# Indicates if the paragraph will have to be translated
+	my $translate = 0;
+
 	my ($eof,@paragraph)=$self->get_string_until('<',{remove=>1});
 
-	while (!$eof and !$self->breaking_tag) {
-		my @text;
-		# Append the found inline tag
-		($eof,@text)=$self->get_string_until('>',{include=>1,remove=>1,unquoted=>1});
-		push @paragraph, @text;
+	# Check if this has to be translated
+	if ($self->join_lines(@paragraph) !~ /^\s*$/s) {
+		my $struc = $self->get_path;
+		my $inlist = 0;
+		if ($self->tag_in_list($struc,@{$self->{tags}})) {
+			$inlist = 1;
+		}
+		if ($self->{options}{'tagsonly'} eq $inlist) {
+			$translate = 1;
+		}
+	}
 
+	while (!$eof and !$self->breaking_tag) {
+	NEXT_TAG:
+		my @text;
+		my $type = $self->tag_type;
+		my $f_extract = $tag_types[$type]->{'f_extract'};
+		if (    defined($f_extract)
+		    and $f_extract eq \&tag_extract_comment) {
+			# Remove the content of the comments
+			($eof, @text) = $self->extract_tag($type,1);
+		} else {
+			my ($tmpeof, @tag) = $self->extract_tag($type,0);
+			# Append the found inline tag
+			($eof,@text)=$self->get_string_until('>',
+			                                     {include=>1,
+			                                      remove=>1,
+			                                      unquoted=>1});
+			# Append or remove the opening/closing tag from
+			# the tag path
+			if ($tag_types[$type]->{'end'} eq "") {
+				if ($tag_types[$type]->{'beginning'} eq "") {
+					push @path, $self->get_tag_name(@tag);
+				} elsif ($tag_types[$type]->{'beginning'} eq "/") {
+					my $test = pop @path;
+					if (!defined($test) ||
+					    $test ne $tag[0] ) {
+						die wrap_ref_mod($tag[1], "po4a::xml", dgettext("po4a", "Unexpected closing tag </%s> found. The main document may be wrong."), $tag[0]);
+					}
+				}
+			}
+			push @paragraph, @text;
+		}
+
+		# Next tag
 		($eof,@text)=$self->get_string_until('<',{remove=>1});
 		if ($#text > 0) {
+			# Check if text (extracted after the inline tag)
+			# has to be translated
+			if ($self->join_lines(@text) !~ /^\s*$/s) {
+				my $struc = $self->get_path;
+				my $inlist = 0;
+				if ($self->tag_in_list($struc,
+				                       @{$self->{tags}})) {
+					$inlist = 1;
+				}
+				if ($self->{options}{'tagsonly'} eq $inlist) {
+					$translate = 1;
+				}
+			}
 			push @paragraph, @text;
 		}
+
+		# If the next tag closes the last inline tag, we loop again
+		# (In the case of <foo><bar> being the inline tag, we can't
+		# loop back with the "while" because breaking_tag will check
+		# for <foo><bar><bar>, hence the goto)
+		$type = $self->tag_type;
+		if (    ($tag_types[$type]->{'end'} eq "")
+		    and ($tag_types[$type]->{'beginning'} eq "/") ) {
+			my ($tmpeof, @tag) = $self->extract_tag($type,0);
+			if ($self->get_tag_name(@tag) eq $path[$#path]) {
+				# The next tag closes the last inline tag.
+				# We nned to temporarily remove the tag from
+				# the path before calling breaking_tag
+				my $t = pop @path;
+				if (!$tmpeof and !$self->breaking_tag) {
+					push @path, $t;
+					goto NEXT_TAG;
+				}
+				push @path, $t;
+			}
+		}
 	}
 
 	# This strips the extracted strings
@@ -884,17 +992,8 @@ sub treat_content {
 	if ( length($self->join_lines(@paragraph)) > 0 ) {
 		my $struc = $self->get_path;
 		my $options = $self->tag_in_list($struc,@{$self->{tags}});
-		my $inlist;
-		if ($options eq 0) {
-			$inlist = 0;
-			$options = "";
-		} elsif ($options eq 1) {
-			$inlist = 1;
-			$options = "";
-		} else {
-			$inlist = 1;
-		}
-		if ( $self->{options}{'tagsonly'} eq $inlist ) {
+		$options = "" if ($options eq 0 or $options eq 1);
+		if ($translate) {
 			# This tag should be translated
 			$self->pushline($self->found_string(
 				$self->join_lines(@paragraph),
@@ -949,6 +1048,13 @@ sub treat_options {
 	$self->{options}{'inline'} =~ /\s*(.*)\s*/s;
 	my @list_inline = split(/\s+/s,$1);
 	$self->{inline} = \@list_inline;
+        
+	$self->{options}{'nodefault'} =~ /\s*(.*)\s*/s;
+	my %list_nodefault;
+	foreach (split(/\s+/s,$1)) {
+	    $list_nodefault{$_} = 1;
+	}
+	$self->{nodefault} = \%list_nodefault;
 }
 
 =head2 GETTING TEXT FROM THE INPUT DOCUMENT
@@ -1103,6 +1209,11 @@ The writing of derivate modules is rather limited.
 
 DOCTYPE (ENTITIES)
 
+There is a minimal support for the translation of entities. They are
+translated as a whole, and tags are not taken into account. Multilines
+entities are not supported and entities are always rewrapped during the
+translation.
+
 INCLUDED FILES
 
 MODIFY TAG TYPES FROM INHERITED MODULES
@@ -1116,11 +1227,11 @@ L<po4a(7)|po4a.7>, L<Locale::Po4a::TransTractor(3pm)|Locale::Po4a::TransTractor>
 
 =head1 AUTHORS
 
- Jordi Vilalta <jvprat@wanadoo.es>
+ Jordi Vilalta <jvprat@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2004 by Jordi Vilalta  E<lt>jvprat@wanadoo.esE<gt>
+Copyright (c) 2004 by Jordi Vilalta  E<lt>jvprat@gmail.comE<gt>
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see the COPYING file).
